function LMEA()
% A Decision Variable Clustering-Based Evolutionary Algorithm for
% Large-scale Many-objective Optimization (LMEA), TEVC, 2016
% <Xingyi Zhang, Ye Tian, Yaochu Jin and Ran Cheng>
%
% This is an enhanced version of the original LMEA presented in the paper,
% which can perform better on several test instances

% Copyright 2015-2016 Ye Tian

clc; format compact; tic;
global evaluated;

    %% Parameter Setting
    problem     = 'LSMOP3';  % test problem
    M           = 5;        % number of objectives
    D           = 100;      % length of decision variables
    N           = 126;      % population size
	evaluation  = 1e6;      % number of maximal evaluations
    
    operator    = 'SBX';    % can be set to 'SBX' or 'DE'
    nSel        = 5;        % number of selected solutions in variable clustering
    nPer        = 50;       % number of perturbations in variable clustering
    nCor        = 5;        % number of selected solutions in variable correlation

    %% Generate random population
    evaluated             = 0;
    [Population,Boundary] = F_objective('init',problem,M,N,D);
    ObjValue              = F_objective('value',problem,M,Population);

    %% Detect the group of each distance variable
    [PV,DV] = VariableClustering(Population,ObjValue,nSel,nPer,problem,Boundary);
    DVSet   = CorrelationAnalysis(Population,ObjValue,DV,nCor,problem,Boundary);

    %% Start the iterations
    while evaluated < evaluation
        % Convergence optimization
        for i = 1 : 10
        	[Population,ObjValue] = ConvergenceOptimization(Population,ObjValue,DVSet,operator,problem,Boundary);
        end
        % Distribution optimization
        for i = 1 : 1
            [Population,ObjValue] = DistributionOptimization(Population,ObjValue,PV,problem,Boundary);
        end
        clc;fprintf('LMEA on %5s with %2s-objective, %4s%% completed, %5s seconds past ... ...\n',problem,num2str(M),num2str(roundn(evaluated/evaluation*100,-1)),num2str(roundn(toc,-2)));
    end
    Draw(ObjValue);
end

function [Population,ObjValue] = ConvergenceOptimization(Population,ObjValue,DVSet,operator,problem,Boundary)
    N      = size(Population,1);
    oldCon = calCon(ObjValue);
    for i = 1 : length(DVSet)
        for j = 1 : length(DVSet{i})
            % Select parents
            MatingPool = TournamentSelection(2,2*N,oldCon);
            % Generate offsprings
            if strcmp(operator,'SBX')
                Offspring = VariationSpecial(Population,Population(MatingPool,:),DVSet{i},operator,Boundary);
            else
                Offspring = VariationSpecial(Population,Population([1:N,MatingPool],:),DVSet{i},operator,Boundary);
            end
            OffObjValue = F_objective('value',problem,size(ObjValue,2),Offspring);
            % Update each solution
            allCon  = calCon([ObjValue;OffObjValue]);
            oldCon  = allCon(1:N);
            newCon  = allCon(N+1:end);
            updated = oldCon > newCon;
            Population(updated,:) = Offspring(updated,:);
            ObjValue(updated,:)   = OffObjValue(updated,:);
            oldCon(updated)       = newCon(updated);
        end
    end
end

function [Population,ObjValue] = DistributionOptimization(Population,ObjValue,PV,problem,Boundary)
    N = size(Population,1);
    % Select parents
    Parents    = TournamentSelection(2,N,calCon(ObjValue));
    MatingPool = randi(N,1,ceil(N/2)*2);
    % Generate offsprings
    Offspring   = VariationSpecial(Population(Parents,:),Population(MatingPool,:),PV,'SBX',Boundary);
    OffObjValue = F_objective('value',problem,size(ObjValue,2),Offspring);
    Population  = [Population;Offspring];
    ObjValue    = [ObjValue;OffObjValue];
    % Environmental selection
    Next       = EnvironmentalSelection(ObjValue,N);
    Population = Population(Next,:);
    ObjValue   = ObjValue(Next,:);
end

function Con = calCon(ObjValue)
    FrontNO = ENS(ObjValue,inf);
    Con     = sum(ObjValue,2);
    Con     = FrontNO'*(max(Con)-min(Con)) + Con;
end